@@ -329,6 +329,9 @@ extern const cl_I expt_pos (const cl_I& x, const cl_I& y);
 extern const cl_I factorial (uintL n);
 //CL_REQUIRE(cl_I_factorial)
 
+// Double factorial (!! n), with n Fixnum >=0.  Returns integer.
+extern const cl_I doublefactorial (uintL n);
+
 // Binomialkoeffizient (n \choose k) = n! / k! (n-k)!, wo n,k >= 0 sind.
 extern const cl_I binomial (uintL n, uintL k);
 
