@@ -56,7 +56,7 @@
 
 // OK, stop reading here, because it's getting obscene.
 
-#if defined(__GNUC__) && defined(__OPTIMIZE__) && (!defined(__hppa__) || (__GNUC_MINOR__ >= 8)) && !defined(NO_PROVIDE_REQUIRE)
+#if defined(__GNUC__) && defined(__OPTIMIZE__) && !(defined(__hppa__) && (__GNUC__ == 2) && (__GNUC_MINOR__ < 8)) && !defined(NO_PROVIDE_REQUIRE)
   #ifdef ASM_UNDERSCORE
     #define ASM_UNDERSCORE_PREFIX "_"
   #else
@@ -68,7 +68,7 @@
     // Some m68k systems use "xdef" or "global" or ".global"...
     #define CL_GLOBALIZE_LABEL(label)  __asm__("\t.globl " label);
   #endif
-  #if defined(__sparc__) || defined(__arm__)
+  #if defined(__sparc__) || defined(__sparc64__) || defined(__arm__)
     // Some arm systems use "EXPORT" or ".globl"...
     #define CL_GLOBALIZE_LABEL(label)  __asm__("\t.global " label);
   #endif
@@ -102,7 +102,7 @@
     #define CL_OUTPUT_LABEL(label)  ASM_VOLATILE ("\n" label ":")
   #endif
   // ASM_VOLATILE(string) is for asms without arguments only!!
-  #if (__GNUC__ == 2) && (__GNUC_MINOR__ >= 91)
+  #if ((__GNUC__ == 2) && (__GNUC_MINOR__ >= 91)) || (__GNUC__ >= 3)
     // avoid warning caused by the volatile keyword
     #define ASM_VOLATILE  __asm__
   #else
@@ -122,7 +122,7 @@
   #if defined(__mips__)
     #define CL_JUMP_TO(addr)  ASM_VOLATILE("%*j %0" : : "d" ((void*)(addr)))
   #endif
-  #if defined(__sparc__)
+  #if defined(__sparc__) || defined(__sparc64__)
     #define CL_JUMP_TO(addr)  ASM_VOLATILE("jmp %0\n\tnop" : : "r" ((void*)(addr)))
   #endif
   #if defined(__alpha__)
