@@ -63,7 +63,7 @@ public:
   #define cl_alloca(size)  ((size) >= CL_ALLOCA_MAX ? _alloca_stack.push(cl_alloc_alloca_header(size)) : __builtin_alloca(size))
   #define cl_small_alloca(size)  __builtin_alloca(size)
   #define CL_SMALL_ALLOCA_STACK
-#elif !defined(NO_ALLOCA) && !defined(__sparc__)
+#elif !defined(NO_ALLOCA) && !defined(__sparc__) && !defined(__sparc64__)
   #define cl_alloca(size)  ((size) >= CL_ALLOCA_MAX ? _alloca_stack.push(cl_alloc_alloca_header(size)) : alloca(size))
   #define cl_small_alloca(size)  alloca(size)
   #define CL_SMALL_ALLOCA_STACK
