@@ -10,7 +10,9 @@
 // < sint16 ergebnis: 0 falls wert>=0, -1 falls wert<0.
 inline sint16 sign_of (sint16 wert)
 {
-#if defined(__sparc__) || defined(__arm__)
+#if defined(__sparc64__)
+	return (sint64)wert >> 63;
+#elif defined(__sparc__) || defined(__arm__)
 	return (sint32)wert >> 31;
 #else
 	return (wert >= 0 ? 0 : -1);
@@ -23,7 +25,9 @@ inline sint16 sign_of (sint16 wert)
 // < sint32 ergebnis: 0 falls wert>=0, -1 falls wert<0.
 inline sint32 sign_of (sint32 wert)
 {
-#if defined(__sparc__) || defined(__arm__)
+#if defined(__sparc64__)
+	return (sint64)wert >> 63;
+#elif defined(__sparc__) || defined(__arm__)
 	return wert >> 31;
 #else
 	return (wert >= 0 ? 0 : -1);
@@ -109,7 +113,7 @@ inline uint64 highlow64_0 (uint32 high)
 // mulu16(arg1,arg2)
 // > arg1, arg2 : zwei 16-Bit-Zahlen
 // < ergebnis: eine 32-Bit-Zahl
-#if defined(__GNUC__) && defined(__sparc__) && defined(FAST_DOUBLE)
+#if defined(__GNUC__) && defined(__sparc__) && !defined(__sparc64__) && defined(FAST_DOUBLE)
 // Ist das schneller als mulu16_ ??
 inline uint32 mulu16 (uint16 arg1, uint16 arg2)
 {
@@ -118,6 +122,16 @@ inline uint32 mulu16 (uint16 arg1, uint16 arg2)
 		 + (double)(4503599627370496.0L); // + 2^52, zum Normalisieren
 	return __fi.i[1]; // untere 32 Bit herausholen (benutzt CL_CPU_BIG_ENDIAN_P !)
 }
+#elif defined(__GNUC__) && defined(__sparc64__) && !defined(NO_ASM)
+inline uint32 mulu16 (uint16 arg1, uint16 arg2)
+{
+	register uint64 _prod;
+	__asm__("umul %1,%2,%0"
+		: "=r" (_prod)
+		: "r" (arg1), "r" (arg2)
+	       );
+	return _prod;
+}
 #elif defined(__GNUC__) && defined(__i386__) && !defined(NO_ASM)
 inline uint32 mulu16 (uint16 arg1, uint16 arg2)
 {
@@ -129,7 +143,7 @@ inline uint32 mulu16 (uint16 arg1, uint16 arg2)
 	       );
 	return highlow32(_hi,_lo);
 }
-#elif defined(__sparc__)
+#elif defined(__sparc__) || defined(__sparc64__)
   extern "C" uint32 mulu16_ (uint16 arg1, uint16 arg2);
   #define mulu16  mulu16_  // extern in Assembler
 #else
@@ -143,7 +157,7 @@ inline uint32 mulu16 (uint16 arg1, uint16 arg2)
 // mulu24(arg1,arg2,hi=,lo=);
 // > arg1, arg2 : zwei 24-Bit-Zahlen
 // < 2^32*hi+lo : eine 48-Bit-Zahl
-#if defined(__sparc__) && defined(FAST_DOUBLE)
+#if defined(__sparc__) && !defined(__sparc64__) && defined(FAST_DOUBLE)
   #define mulu24(x,y,hi_zuweisung,lo_zuweisung)  \
     { var uint32 _x = (x);					\
       var uint32 _y = (y);					\
@@ -162,7 +176,17 @@ inline uint32 mulu16 (uint16 arg1, uint16 arg2)
 // > arg1, arg2 : zwei 32-Bit-Zahlen
 // < ergebnis : eine 32-Bit-Zahl
 // Es wird vorausgesetzt, da arg1*arg2 < 2^32.
-#if defined(__sparc__)
+#if defined(__GNUC__) && defined(__sparc64__) && !defined(NO_ASM)
+inline uint32 mulu32_unchecked (uint32 arg1, uint32 arg2)
+{
+	register uint64 _prod;
+	__asm__("umul %1,%2,%0"
+		: "=r" (_prod)
+		: "r" (arg1), "r" (arg2)
+	       );
+	return _prod;
+}
+#elif defined(__sparc__)
   extern "C" uint32 mulu32_unchecked (uint32 x, uint32 y); // extern in Assembler
 #else
   // Wir knnen dafr auch die Bibliotheksroutine des C-Compilers nehmen:
@@ -209,6 +233,16 @@ inline uint32 mulu16 (uint16 arg1, uint16 arg2)
        hi_zuweisung _hi;						\
        lo_zuweisung _lo;						\
      })
+#elif defined(__GNUC__) && defined(__sparc64__) && !defined(NO_ASM)
+  #define mulu32(x,y,hi_zuweisung,lo_zuweisung)  \
+    ({ var register uint64 _hi;					\
+       var register uint64 _lo;					\
+       __asm__("umul %2,%3,%1\n\trd %y,%0"			\
+	       : "=r" (_hi), "=r" (_lo)				\
+	       : "r" ((uint32)(x)), "r" ((uint32)(y))		\
+	      );						\
+       hi_zuweisung (uint32)_hi; lo_zuweisung (uint32)_lo;	\
+     })
 #elif defined(__GNUC__) && defined(__sparc__)
   #define mulu32(x,y,hi_zuweisung,lo_zuweisung)  \
     ({ lo_zuweisung mulu32_(x,y); /* extern in Assembler */	\
@@ -260,9 +294,9 @@ inline uint32 mulu16 (uint16 arg1, uint16 arg2)
 #else
   #define mulu32(x,y,hi_zuweisung,lo_zuweisung)  \
     { lo_zuweisung mulu32_(x,y); hi_zuweisung mulu32_high; }
-  #if defined(__m68k__) || defined(__sparc__) || defined(__arm__) || (defined(__i386__) && !defined(WATCOM) && !defined(MICROSOFT)) || defined(__mips__) || defined(__hppa__)
+  #if defined(__m68k__) || defined(__sparc__) || defined(__sparc64__) || defined(__arm__) || (defined(__i386__) && !defined(WATCOM) && !defined(MICROSOFT)) || defined(__mips__) || defined(__hppa__)
     // mulu32_ extern in Assembler
-    #if defined(__sparc__)
+    #if defined(__sparc__) || defined(__sparc64__)
       extern "C" uint32 _get_g1 (void);
       #define mulu32_high  (_get_g1()) // Rckgabe im Register %g1
     #elif !defined(__hppa__)
@@ -309,10 +343,26 @@ inline uint32 mulu16 (uint16 arg1, uint16 arg2)
        hi_zuweisung _hi;		\
        lo_zuweisung _lo;		\
      })
+#elif defined(__GNUC__) && defined(__sparc64__)
+  #define mulu64(x,y,hi_zuweisung,lo_zuweisung)  \
+    ({ lo_zuweisung mulu64_(x,y); /* extern in Assembler */	\
+      {var register uint64 _hi __asm__("%g2");			\
+       hi_zuweisung _hi;					\
+     }})
 #else
   #define mulu64(x,y,hi_zuweisung,lo_zuweisung)  \
     { lo_zuweisung mulu64_(x,y); hi_zuweisung mulu64_high; }
-  #define NEED_FUNCTION_mulu64_
+  #if defined(__sparc64__)
+    // mulu64_ extern in Assembler
+    #if defined(__sparc64__)
+      extern "C" uint64 _get_g2 (void);
+      #define mulu64_high  (_get_g2()) // Rckgabe im Register %g2
+    #else
+      #define NEED_VAR_mulu64_high
+    #endif
+  #else
+    #define NEED_FUNCTION_mulu64_
+  #endif
 #endif
 
 #endif /* HAVE_FAST_LONGLONG */
@@ -348,7 +398,23 @@ inline uint32 mulu16 (uint16 arg1, uint16 arg2)
   extern "C" uint16 divu_3216_1616_ (uint32 x, uint16 y); // -> Quotient q
   extern "C" uint16 divu_16_rest;                         // -> Rest r
 #endif
-#if defined(__GNUC__) && defined(__sparc__)
+#if defined(__GNUC__) && defined(__sparc64__) && !defined(NO_ASM)
+  #define divu_3216_1616(x,y,q_zuweisung,r_zuweisung)  \
+    ({var uint32 __x = (x);        \
+      var uint16 __y = (y);        \
+      var uint64 __q;              \
+      var uint64 __r;              \
+      __asm__ __volatile__ (       \
+        "wr %%g0,%%g0,%%y\n\t"     \
+        "udiv %2,%3,%0\n\t"        \
+        "umul %0,%3,%1"            \
+        "sub %2,%1,%1"             \
+        : "=&r" (__q), "=&r" (__r) \
+        : "r" (__x), "r" (__y));   \
+      q_zuweisung (uint16)__q;     \
+      r_zuweisung (uint16)__r;     \
+     })
+#elif defined(__GNUC__) && (defined(__sparc__) || defined(__sparc64__))
   #define divu_3216_1616(x,y,q_zuweisung,r_zuweisung)  \
     ({ var uint32 __qr = divu_3216_1616_(x,y); /* extern in Assembler */\
        q_zuweisung low16(__qr);						\
@@ -392,7 +458,7 @@ inline uint32 mulu16 (uint16 arg1, uint16 arg2)
       q_zuweisung __q;							\
       r_zuweisung (__x - __q * __y);					\
      })
-#elif defined(__sparc__)
+#elif defined(__sparc__) || defined(__sparc64__)
   #define divu_3216_1616(x,y,q_zuweisung,r_zuweisung)  \
     { var uint32 __qr = divu_3216_1616_(x,y); /* extern in Assembler */	\
       q_zuweisung low16(__qr);						\
@@ -421,7 +487,23 @@ inline uint32 mulu16 (uint16 arg1, uint16 arg2)
 // < x = q*y+r
   extern "C" uint32 divu_3216_3216_ (uint32 x, uint16 y); // -> Quotient q
   extern "C" uint16 divu_16_rest;                         // -> Rest r
-#if defined(__sparc__) || defined(__i386__)
+#if defined(__GNUC__) && defined(__sparc64__) && !defined(NO_ASM)
+  #define divu_3216_3216(x,y,q_zuweisung,r_zuweisung)  \
+    ({var uint32 __x = (x);        \
+      var uint16 __y = (y);        \
+      var uint64 __q;              \
+      var uint64 __r;              \
+      __asm__ __volatile__ (       \
+        "wr %%g0,%%g0,%%y\n\t"     \
+        "udiv %2,%3,%0\n\t"        \
+        "umul %0,%3,%1"            \
+        "sub %2,%1,%1"             \
+        : "=&r" (__q), "=&r" (__r) \
+        : "r" (__x), "r" (__y));   \
+      q_zuweisung (uint32)__q;     \
+      r_zuweisung (uint16)__r;     \
+     })
+#elif defined(__sparc__) || defined(__sparc64__) || defined(__i386__)
   #define divu_3216_3216  divu_3232_3232
 #else
   // Methode: (beta = 2^16)
@@ -455,7 +537,23 @@ inline uint32 mulu16 (uint16 arg1, uint16 arg2)
 // < x = q*y+r
   extern "C" uint32 divu_3232_3232_ (uint32 x, uint32 y); // -> Quotient q
   extern "C" uint32 divu_32_rest;                         // -> Rest r
-#if defined(__sparc__) || defined(__i386__)
+#if defined(__GNUC__) && defined(__sparc64__) && !defined(NO_ASM)
+  #define divu_3232_3232(x,y,q_zuweisung,r_zuweisung)  \
+    ({var uint32 __x = (x);        \
+      var uint32 __y = (y);        \
+      var uint64 __q;              \
+      var uint64 __r;              \
+      __asm__ __volatile__ (       \
+        "wr %%g0,%%g0,%%y\n\t"     \
+        "udiv %2,%3,%0\n\t"        \
+        "umul %0,%3,%1"            \
+        "sub %2,%1,%1"             \
+        : "=&r" (__q), "=&r" (__r) \
+        : "r" (__x), "r" (__y));   \
+      q_zuweisung (uint32)__q;     \
+      r_zuweisung (uint32)__r;     \
+     })
+#elif defined(__sparc__) || defined(__sparc64__) || defined(__i386__)
   #define divu_3232_3232(x,y,q_zuweisung,r_zuweisung)  \
     divu_6432_3232(0,x,y,q_zuweisung,r_zuweisung)
   #define divu_3232_3232_(x,y) divu_6432_3232_(0,x,y)
@@ -550,7 +648,24 @@ inline uint32 mulu16 (uint16 arg1, uint16 arg2)
      })
   #define divu_6432_3232_(xhi,xlo,y) \
     ({var uint32 ___q; divu_6432_3232(xhi,xlo,y,___q=,); ___q; })
-#elif defined(__GNUC__) && defined(__sparc__)
+#elif defined(__GNUC__) && defined(__sparc64__) && !defined(NO_ASM)
+  #define divu_6432_3232(xhi,xlo,y,q_zuweisung,r_zuweisung)  \
+    ({var uint32 __xhi = (xhi);    \
+      var uint32 __xlo = (xlo);    \
+      var uint32 __y = (y);        \
+      var uint64 __q;              \
+      var uint64 __r;              \
+      __asm__ __volatile__ (       \
+        "wr %2,%%g0,%%y\n\t"       \
+        "udiv %3,%4,%0\n\t"        \
+        "umul %0,%4,%1"            \
+        "sub %3,%1,%1"             \
+        : "=&r" (__q), "=&r" (__r) \
+        : "r" (__xhi), "r" (__xlo), "r" (__y)); \
+      q_zuweisung (uint32)__q;     \
+      r_zuweisung (uint32)__r;     \
+     })
+#elif defined(__GNUC__) && (defined(__sparc__) || defined(__sparc64__))
   #define divu_6432_3232(xhi,xlo,y,q_zuweisung,r_zuweisung)  \
     ({ var uint32 _q = divu_6432_3232_(xhi,xlo,y); /* extern in Assembler */\
        var register uint32 _r __asm__("%g1");				    \
@@ -607,9 +722,9 @@ inline uint32 mulu16 (uint16 arg1, uint16 arg2)
 #else
   #define divu_6432_3232(xhi,xlo,y,q_zuweisung,r_zuweisung)  \
     { q_zuweisung divu_6432_3232_(xhi,xlo,y); r_zuweisung divu_32_rest; }
-  #if defined(__m68k__) || defined(__sparc__) || defined(__arm__) || (defined(__i386__) && !defined(WATCOM) && !defined(MICROSOFT)) || defined(__hppa__)
+  #if defined(__m68k__) || defined(__sparc__) || defined(__sparc64__) || defined(__arm__) || (defined(__i386__) && !defined(WATCOM) && !defined(MICROSOFT)) || defined(__hppa__)
     // divu_6432_3232_ extern in Assembler
-    #if defined(__sparc__)
+    #if defined(__sparc__) || defined(__sparc64__)
       extern "C" uint32 _get_g1 (void);
       #define divu_32_rest  (_get_g1()) // Rckgabe im Register %g1
     #else
@@ -722,7 +837,7 @@ inline uint32 mulu16 (uint16 arg1, uint16 arg2)
 // > uint32 xhi,xlo: Radikand x = 2^32*xhi+xlo, >= 2^62, < 2^64
 // < uint32 y: floor(sqrt(x)), >= 2^31, < 2^32
 // < boolean sqrtp: /=0, falls x=y^2
-#if defined(__sparc__) || defined(__m68k__) || defined(__hppa__)
+#if defined(__sparc__) || defined(__sparc64__) || defined(__m68k__) || defined(__hppa__)
   // Methode:
   // y := 2^32 als Anfangswert,
   // y := floor((y + floor(x/y))/2) als nchster Wert,
@@ -919,7 +1034,7 @@ inline uint32 mulu16 (uint16 arg1, uint16 arg2)
       __asm__("bfffo %1{#0:#8},%0" : "=d" (zero_counter) : "dm" ((uint8)(digit)) ); \
       size_zuweisung (8-zero_counter);                                              \
     }
-#elif defined(__sparc__)
+#elif defined(__sparc__) && !defined(__sparc64__)
   #define integerlength8(digit,size_zuweisung)  \
     integerlength32((uint32)(digit),size_zuweisung) // siehe unten
 #elif defined(__GNUC__) && defined(__i386__) && !defined(NO_ASM)
@@ -951,7 +1066,7 @@ inline uint32 mulu16 (uint16 arg1, uint16 arg2)
       __asm__("bfffo %1{#0:#16},%0" : "=d" (zero_counter) : "dm" ((uint16)(digit)) ); \
       size_zuweisung (16-zero_counter);                                               \
     }
-#elif defined(__sparc__)
+#elif defined(__sparc__) && !defined(__sparc64__)
   #define integerlength16(digit,size_zuweisung)  \
     integerlength32((uint32)(digit),size_zuweisung) // siehe unten
 #elif defined(__GNUC__) && defined(__i386__) && !defined(NO_ASM)
@@ -995,7 +1110,7 @@ inline uint32 mulu16 (uint16 arg1, uint16 arg2)
       __asm__("bfffo %1{#0:#32},%0" : "=d" (zero_counter) : "dm" ((uint32)(digit)) ); \
       size_zuweisung (32-zero_counter);                                               \
     }
-#elif defined(__sparc__) && defined(FAST_DOUBLE)
+#elif defined(__sparc__) && !defined(__sparc64__) && defined(FAST_DOUBLE)
   #define integerlength32(digit,size_zuweisung)  \
     {var union { double f; uint32 i[2]; } __fi;				\
      const int df_mant_len = 52;  /* mantissa bits (excl. hidden bit) */\
@@ -1116,7 +1231,7 @@ inline uint32 mulu16 (uint16 arg1, uint16 arg2)
         count_zuweisung one_position;                                          \
       }
     #define FAST_ORD2
-  #elif defined(__sparc__)
+  #elif defined(__sparc__) && !defined(__sparc64__)
     #define ord2_32(digit,count_zuweisung)  \
     { var uint32 n = (digit);                                             \
       n = n | -n;                                                         \
