@@ -80,7 +80,7 @@ mulu32_:
         multu $5,$4             // arg1 * arg2
         mfhi $6                 // hi
         mflo $2                 // lo
-        sw $6,mulu32_high       // hi abspeichern # Adressierung?? Deklaration??
+        sw $6,mulu32_high       // hi abspeichern // Adressierung?? Deklaration??
         j $31                   // return
         .end mulu32_
 
@@ -278,7 +278,7 @@ eqv_loop_up:
 nalu1:    lw $12,($4)           // x = *xptr
           lw $13,($5)           // y = *yptr
           addu $5,4             // yptr++
-          and $12,$13           // x &= y        # Gibt es 'nand $12,$13' ??
+          and $12,$13           // x &= y        // Gibt es 'nand $12,$13' ??
           nor $12,$0            // x = ~x
           sw $12,($4)           // *xptr = x
           addu $4,4             // xptr++
@@ -748,7 +748,7 @@ nald1:    subu $4,4             // xptr--
           lw $12,($4)           // x = *xptr
           lw $13,($5)           // y = *yptr
           subu $6,1             // count--
-          and $12,$13           // x &= y        # Gibt es 'nand $12,$13' ??
+          and $12,$13           // x &= y        // Gibt es 'nand $12,$13' ??
           nor $12,$0            // x = ~x
           sw $12,($4)           // *xptr = x
 nand_loop_down:
