@@ -12,7 +12,7 @@
 #include "cl_base_config.h"
 #include "cl_low.h"
 
-#if defined(unix) || defined(__unix) || defined(_AIX) || defined(sinix)
+#if defined(unix) || defined(__unix) || defined(_AIX) || defined(sinix) || (defined(_WIN32) && defined(__GNUC__))
 
 #include <sys/types.h>
 #include <unistd.h> // declares getpid()
@@ -37,7 +37,11 @@ inline uint32 get_seed (void)
 #elif defined(HAVE_FTIME)
 
 #include <sys/timeb.h>
-extern "C" int ftime (struct timeb * tp);
+#ifdef _WIN32
+  extern "C" void ftime (struct timeb * tp);
+#else
+  extern "C" int ftime (struct timeb * tp);
+#endif
 
 inline uint32 get_seed (void)
 {
@@ -53,7 +57,7 @@ inline uint32 get_seed (void)
 #include <sys/time.h>
 #endif
 #include <sys/times.h>
-extern "C" CLOCK_T times (struct tms * buffer);
+extern "C" clock_t times (struct tms * buffer);
 
 inline uint32 get_seed (void)
 {
@@ -74,7 +78,7 @@ cl_random_state::cl_random_state ()
 {
 	var uint32 seed_hi;
 	var uint32 seed_lo;
-#if defined(unix) || defined(__unix) || defined(_AIX) || defined(sinix)
+#if defined(unix) || defined(__unix) || defined(_AIX) || defined(sinix) || (defined(_WIN32) && defined(__GNUC__))
 	seed_lo = get_seed();
 	seed_hi = (rand() // zufllige 31 Bit (bei UNIX_BSD) bzw. 16 Bit (bei UNIX_SYSV)
                           << 8) ^ (uintL)(getpid()); // ca. 8 Bit von der Process ID
