@@ -284,7 +284,7 @@ inline const cl_I minus (uintL x, uintL y)
 #if (intDsize<=32)
 
 // Holt die nchsten pFN_maxlength Digits in ein uint32.
-inline uint32 pFN_maxlength_digits_at (uintD* ptr)
+inline uint32 pFN_maxlength_digits_at (const uintD* ptr)
 {
 #if (pFN_maxlength==1)
 	return (uint32)lspref(ptr,0);
@@ -317,6 +317,14 @@ inline void set_pFN_maxlength_digits_at (uintD* ptr, uint32 wert)
 #endif
 }
 
+#elif (intDsize==64)
+
+// Holt die nchsten pFN_maxlength Digits in ein uint64.
+inline uint64 pFN_maxlength_digits_at (const uintD* ptr)
+{
+	return (uint64)lspref(ptr,0);
+}
+
 #endif
 
 
