@@ -11,15 +11,19 @@
 
 #include "cl_N.h"
 
+// Cf. cl_I_p in cl_I_ring.cc.
+// But here, for better inlining in g++, it is preferrable to finish every
+// alternative with either "return cl_true;" or "return cl_false;".
+
 inline cl_boolean cl_I_p (const cl_number& x)
 {
 	if (!x.pointer_p())
-		switch (cl_tag((x).word)) {
+		switch (x.nonpointer_tag()) {
 		case cl_FN_tag:
 			return cl_true;
 		}
 	else
-		if (x.heappointer->type == &cl_class_bignum)
+		if (x.pointer_type() == &cl_class_bignum)
 			return cl_true;
 	return cl_false;
 }
