@@ -94,7 +94,7 @@ static const _cl_UP gf2_uminus (cl_heap_univpoly_ring* UPR, const _cl_UP& x)
 	return x;
 }
 
-#if !defined(__sparc__)
+#if !(defined(__sparc__) || defined(__sparc64__))
 // Multiplication of polynomials over GF(2) can unfortunately not profit
 // from hardware multiply instructions. Use a table instead.
 // This is a 2^8 x 2^4 table. Maybe a 2^6 x 2^6 table would be better?
@@ -624,7 +624,7 @@ static uint16 gf2_mul_table[0x100][0x10] =
 #endif
 
 // Multiply two GF2-polynomials of degree < 16.
-#if defined(__sparc__)
+#if defined(__sparc__) || defined(__sparc64__)
 extern "C" uint32 gf2_mul16 (uint16 x, uint16 y);
 #else
 uint32 gf2_mul16 (uint16 x, uint16 y)
@@ -648,7 +648,7 @@ uint32 gf2_mul16 (uint16 x, uint16 y)
 
 // Multiply two GF2-polynomials of degree < 32.
 // Stores the low part, returns the high part.
-#if defined(__sparc__)
+#if defined(__sparc__) || defined(__sparc64__)
 extern "C" uint32 gf2_mul32 (uint32 x, uint32 y, uint32* plo);
 #else
 uint32 gf2_mul32 (uint32 x, uint32 y, uint32* plo)
