@@ -32,7 +32,7 @@ static void uninitialized_error (const _cl_UP& obj)
 	cl_abort();
 }
 
-#if (defined(__sparc__) && !defined(__GNUC__))
+#if ((defined(__sparc__) || defined(__sparc64__)) && !defined(__GNUC__))
   // avoid Sun C++ 4.1 compiler bug
   #define RETDUMMY  return *(_cl_UP*)R
 #else
@@ -83,7 +83,7 @@ static cl_boolean dummy_zerop (cl_heap_univpoly_ring* R, const _cl_UP& x)
 static const _cl_UP dummy_canonhom (cl_heap_univpoly_ring* R, const cl_I& x)
 {
 	unused R;
-	&x; // unused x;
+	(void)&x; // unused x;
 	uninitialized_ring(); RETDUMMY;
 }
 #define dummy_mul dummy_op2
@@ -91,7 +91,7 @@ static const _cl_UP dummy_canonhom (cl_heap_univpoly_ring* R, const cl_I& x)
 static const _cl_UP dummy_expt_pos (cl_heap_univpoly_ring* R, const _cl_UP& x, const cl_I& y)
 {
 	unused R;
-	&y; // unused y;
+	(void)&y; // unused y;
 	uninitialized_error(x); RETDUMMY;
 }
 
